/*=============================================================================
 * DBLP Benchmark. Self-Join and Selection for Entity-Attribute-Value-Table.
 *
 * Benchmark IDs: dblp_dync_*
 * This version uses load_dync and no index for the database relation att.
 *
 * Test results will be recorded in file 'RUN_BENCH__RESULT'
 * This file is a modified version of a file distributed with OpenRuleBench.
 *=============================================================================
 */


/*-----------------------------------------------------------------------------
 * Options:
 *-----------------------------------------------------------------------------
 */

/* Data load method: */
:- import load_dync/2 from consult.
load_data(Filename) :- load_dync(Filename, a).
/* load_data(Filename) :- load_dyn(Filename). */

/* Index: */
/* :- index(att/3, trie). */

/*-----------------------------------------------------------------------------
 * Benchmark Program:
 *-----------------------------------------------------------------------------
 */

queryrule(X,T,Y,A,M) :-
	att(X,title,T),
	att(X,year,Y),
	att(X,author,A),
	att(X,month,M).


/*-----------------------------------------------------------------------------
 * Benchmark Query:
 *-----------------------------------------------------------------------------
 */

bench :- queryrule(_,_,_,_,_), fail.
bench.


/*-----------------------------------------------------------------------------
 * List of Test Data Files:
 *-----------------------------------------------------------------------------
 */

test_file(dblp_dync_std, '../data/dblp.data').
test_file(dblp_dync_big, '../data/dblp_big.data').


/*=============================================================================
 * The rest is the same for all XSB benchmarks:
 *=============================================================================
 */


/*-----------------------------------------------------------------------------
 * Main Predicate: Run the Test and Write File RUN__BENCH_RESULT.
 *-----------------------------------------------------------------------------
 */

test(BenchID) :- 
	test_file(BenchID, Filename),
	open('RUN_BENCH__RESULT', write, Output),
	% Write Benchmark ID to output file:
	write(Output, BenchID),
	write(Output, '\t'),
	% Get times before Load (both times are in s):
	walltime(Real0),
	cputime(CPU0),
	% Load Data:
	load_data(Filename),
	% Get times after Load:
	cputime(CPU1),
	walltime(Real1),
	% Compute Load times:
	LoadCPU is floor((CPU1 - CPU0) * 1000),
	LoadReal is floor((Real1 - Real0) * 1000),
	% Write load CPU time to output file:
	write(Output, LoadCPU),
	write(Output, '\t'),
	flush_output(Output),
	% Get times before Exec:
	walltime(Real3),
	cputime(CPU3),
	% NOW THE BENCHMARK QUERY:
	bench,
	% Get times after Exec:
	cputime(CPU4),
	walltime(Real4),
	% Compute Times for Benchmark Execution:
	ExecCPU is floor((CPU4 - CPU3) * 1000),
	ExecReal is floor((Real4 - Real3) * 1000),
	% Write Exec CPU time to output file:
	write(Output, ExecCPU),
	write(Output, '\t'),
	% Write total CPU time to output file:
	TotalCPU is floor(CPU4 * 1000),
	write(Output, TotalCPU),
	write(Output, '\t'),
	% Compare CPU times with real times, write "Warn" status if strange:
	status(LoadCPU, LoadReal, ExecCPU, ExecReal, Status),
	write(Output, Status),
	write(Output, '\t'),
	close(Output).

/*-----------------------------------------------------------------------------
 * Comparison of Wallclock Time and CPU Time:
 *-----------------------------------------------------------------------------
 */

/* If the two times differ to much, we output status 'Warn': */

status(LoadCPU, LoadReal, _ExecCPU, _ExecReal, 'Warn1') :-
	LoadCPU > LoadReal,
	!.
status(LoadCPU, LoadReal, _ExecCPU, _ExecReal, 'Warn2') :-
	LoadReal > LoadCPU * 1.15,
	!.
status(_LoadCPU, _LoadReal, ExecCPU, ExecReal, 'Warn3') :-
	ExecCPU > ExecReal,
	!.
status(_LoadCPU, _LoadReal, ExecCPU, ExecReal, 'Warn4') :-
	ExecReal > ExecCPU * 1.15,
	!.
status(_LoadCPU, _LoadReal, _ExecCPU, _ExecReal, 'OK').

