/*=============================================================================
 * Tree structure join operation: Join1 Benchmark with Query a(X,Y).
 *
 * Benchmark IDs: j1axy_all_*
 * This version tables all derived predicates (with "subsumptive tabling").
 * In this way, duplicates are already eliminated for the intermediate results.
 *
 * Test results will be recorded in file 'RUN_BENCH__RESULT'
 * This file is a modified version of a file distributed with OpenRuleBench.
 *=============================================================================
 */


/*-----------------------------------------------------------------------------
 * Options:
 *-----------------------------------------------------------------------------
 */

/* Data load method: */
:- import load_dync/2 from consult.
load_data(Filename) :- load_dync(Filename, a).
/* load_data(Filename) :- load_dyn(Filename). */

/* Tabling method: */
:- use_subsumptive_tabling c1/2.
:- use_subsumptive_tabling b1/2.
:- use_subsumptive_tabling b2/2.
:- use_subsumptive_tabling a/2.


/*-----------------------------------------------------------------------------
 * Benchmark Program:
 *-----------------------------------------------------------------------------
 */

a(X, Y)  :- b1(X, Z), b2(Z, Y).

b1(X, Y) :- c1(X, Z), c2(Z, Y).
b2(X, Y) :- c3(X, Z), c4(Z, Y).

c1(X, Y) :- d1(X, Z), d2(Z, Y).


/*-----------------------------------------------------------------------------
 * Benchmark Query:
 *-----------------------------------------------------------------------------
 */

bench :- a(_,_), fail.
bench.


/*-----------------------------------------------------------------------------
 * List of Test Data Files:
 *-----------------------------------------------------------------------------
 */

test_file(j1axy_all_10k, '../data/d1000_relsize10000_xsb_cyc.P').
test_file(j1axy_all_50k, '../data/d1000_relsize50000_xsb_cyc.P').
test_file(j1axy_all_250k, '../data/d1000_relsize250000_xsb_cyc.P').


/*=============================================================================
 * The rest is the same for all XSB benchmarks:
 *=============================================================================
 */


/*-----------------------------------------------------------------------------
 * Main Predicate: Run the Test and Write File RUN__BENCH_RESULT.
 *-----------------------------------------------------------------------------
 */

test(BenchID) :- 
	test_file(BenchID, Filename),
	open('RUN_BENCH__RESULT', write, Output),
	% Write Benchmark ID to output file:
	write(Output, BenchID),
	write(Output, '\t'),
	% Get times before Load (both times are in s):
	walltime(Real0),
	cputime(CPU0),
	% Load Data:
	load_data(Filename),
	% Get times after Load:
	cputime(CPU1),
	walltime(Real1),
	% Compute Load times:
	LoadCPU is floor((CPU1 - CPU0) * 1000),
	LoadReal is floor((Real1 - Real0) * 1000),
	% Write load CPU time to output file:
	write(Output, LoadCPU),
	write(Output, '\t'),
	flush_output(Output),
	% Get times before Exec:
	walltime(Real3),
	cputime(CPU3),
	% NOW THE BENCHMARK QUERY:
	bench,
	% Get times after Exec:
	cputime(CPU4),
	walltime(Real4),
	% Compute Times for Benchmark Execution:
	ExecCPU is floor((CPU4 - CPU3) * 1000),
	ExecReal is floor((Real4 - Real3) * 1000),
	% Write Exec CPU time to output file:
	write(Output, ExecCPU),
	write(Output, '\t'),
	% Write total CPU time to output file:
	TotalCPU is floor(CPU4 * 1000),
	write(Output, TotalCPU),
	write(Output, '\t'),
	% Compare CPU times with real times, write "Warn" status if strange:
	status(LoadCPU, LoadReal, ExecCPU, ExecReal, Status),
	write(Output, Status),
	write(Output, '\t'),
	close(Output).

/*-----------------------------------------------------------------------------
 * Comparison of Wallclock Time and CPU Time:
 *-----------------------------------------------------------------------------
 */

/* If the two times differ to much, we output status 'Warn': */

status(LoadCPU, LoadReal, _ExecCPU, _ExecReal, 'Warn1') :-
	LoadCPU > LoadReal,
	!.
status(LoadCPU, LoadReal, _ExecCPU, _ExecReal, 'Warn2') :-
	LoadReal > LoadCPU * 1.15,
	!.
status(_LoadCPU, _LoadReal, ExecCPU, ExecReal, 'Warn3') :-
	ExecCPU > ExecReal,
	!.
status(_LoadCPU, _LoadReal, ExecCPU, ExecReal, 'Warn4') :-
	ExecReal > ExecCPU * 1.15,
	!.
status(_LoadCPU, _LoadReal, _ExecCPU, _ExecReal, 'OK').

